@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2014 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2018 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -23,22 +23,36 @@
 
 #include <stdlib.h>
 #include <string.h>
+#if HAVE_STDBOOL_H
+#include <stdbool.h>
+#endif
 
-#if HAVE_STDINT_H
-#include <stdint.h>
-#elif HAVE_INTTYPES_H
+#if HAVE_INTTYPES_H
 #include <inttypes.h>
+#elif HAVE_STDINT_H
+#include <stdint.h>
+#endif
+#if HAVE_SYS_TYPES_H
+#include <sys/types.h>
 #endif
 
 #ifndef SNDFILE_H
 #include "sndfile.h"
 #endif
 
+#include <math.h>
+
+#ifdef USE_SSE2
+#include <immintrin.h>
+#endif
+
 #ifdef __cplusplus
 #error "This code is not designed to be compiled with a C++ compiler."
 #endif
 
-#if (SIZEOF_LONG == 8)
+#ifdef INT64_C
+#	define	SF_PLATFORM_S64(x)		INT64_C (x)
+#elif (SIZEOF_LONG == 8)
 #	define	SF_PLATFORM_S64(x)		x##l
 #elif (SIZEOF_LONG_LONG == 8)
 #	define	SF_PLATFORM_S64(x)		x##ll
@@ -74,7 +88,6 @@
 #define	SF_FILENAME_LEN			(1024)
 #define SF_SYSERR_LEN			(256)
 #define SF_MAX_STRINGS			(32)
-#define	SF_HEADER_LEN			(12292)
 #define	SF_PARSELOG_LEN			(2048)
 
 #define	PSF_SEEK_ERROR			((sf_count_t) -1)
@@ -89,30 +102,16 @@
 
 #define		ARRAY_LEN(x)	((int) (sizeof (x) / sizeof ((x) [0])))
 
-#define		NOT(x)			(! (x))
-
-#if COMPILER_IS_GCC
-#define		SF_MAX(x, y)	({ \
-								typeof (x) sf_max_x1 = (x) ; \
-								typeof (y) sf_max_y1 = (y) ; \
-								(void) (&sf_max_x1 == &sf_max_y1) ; \
-								sf_max_x1 > sf_max_y1 ? sf_max_x1 : sf_max_y1 ; })
-
-#define		SF_MIN(x, y)	({ \
-								typeof (x) sf_min_x2 = (x) ; \
-								typeof (y) sf_min_y2 = (y) ; \
-								(void) (&sf_min_x2 == &sf_min_y2) ; \
-								sf_min_x2 < sf_min_y2 ? sf_min_x2 : sf_min_y2 ; })
-#else
 #define		SF_MAX(a, b)	((a) > (b) ? (a) : (b))
 #define		SF_MIN(a, b)	((a) < (b) ? (a) : (b))
-#endif
 
 
 #define		COMPILE_TIME_ASSERT(e)	(sizeof (struct { int : - !! (e) ; }))
 
 
 #define		SF_MAX_CHANNELS		1024
+/* Max FLAC sample rate : https://xiph.org/flac/format.html */
+#define		SF_MAX_SAMPLERATE	655350
 
 
 /*
@@ -123,6 +122,33 @@
 #define SF_CODEC(x)			((x) & SF_FORMAT_SUBMASK)
 #define SF_ENDIAN(x)		((x) & SF_FORMAT_ENDMASK)
 
+/*
+**	Binheader cast macros.
+*/
+
+#define BHW1(x) ((uint8_t) (x))
+#define BHW2(x) ((uint16_t) (x))
+#define BHW3(x) ((uint32_t) (x))
+#define BHW4(x) ((uint32_t) (x))
+#define BHW8(x) ((uint64_t) (x))
+
+#define BHWm(x) ((uint32_t) (x))
+#define BHWS(x) ((char *) (x))
+
+#define BHWf(x) ((double) (x))
+#define BHWd(x) ((double) (x))
+
+#define BHWh(x) ((void *) (x))
+#define BHWj(x) ((size_t) (x))
+#define BHWp(x) ((char *) (x))
+#define BHWo(x) ((size_t) (x))
+#define BHWs(x) ((char *) (x))
+#define BHWv(x) ((const void *) (x))
+#define BHWz(x) ((size_t) (x))
+
+/*------------------------------------------------------------------------------
+*/
+
 enum
 {	/* PEAK chunk location. */
 	SF_PEAK_START		= 42,
@@ -227,7 +253,7 @@ typedef struct
 } STR_DATA ;
 
 typedef struct
-{	int64_t		hash ;
+{	uint64_t	hash ;
 	char		id [64] ;
 	unsigned	id_size ;
 	uint32_t	mark32 ;
@@ -236,7 +262,7 @@ typedef struct
 } READ_CHUNK ;
 
 typedef struct
-{	int64_t		hash ;
+{	uint64_t	hash ;
 	uint32_t	mark32 ;
 	uint32_t	len ;
 	void		*data ;
@@ -264,12 +290,18 @@ struct SF_CHUNK_ITERATOR
 static inline size_t
 make_size_t (int x)
 {	return (size_t) x ;
-} /* size_t_of_int */
+} /* make_size_t */
 
 typedef SF_BROADCAST_INFO_VAR (16 * 1024) SF_BROADCAST_INFO_16K ;
 
 typedef SF_CART_INFO_VAR (16 * 1024) SF_CART_INFO_16K ;
 
+typedef struct
+{	sf_count_t	offset ;
+	sf_count_t	len ;
+	unsigned	minor_version ;
+} ID3V2_HEADER_INFO ;
+
 #if SIZEOF_WCHAR_T == 2
 typedef wchar_t	sfwchar_t ;
 #else
@@ -277,20 +309,13 @@ typedef int16_t sfwchar_t ;
 #endif
 
 
-static inline void *
-psf_memdup (const void *src, size_t n)
-{	void * mem = calloc (1, n & 3 ? n + 4 - (n & 3) : n) ;
-	return memcpy (mem, src, n) ;
-} /* psf_memdup */
+void *psf_memdup (const void *src, size_t n) ;
 
 /*
 **	This version of isprint specifically ignores any locale info. Its used for
 **	determining which characters can be printed in things like hexdumps.
 */
-static inline int
-psf_isprint (int ch)
-{	return (ch >= ' ' && ch <= '~') ;
-} /* psf_isprint */
+int psf_isprint (int ch) ;
 
 /*=======================================================================================
 **	SF_PRIVATE stuct - a pointer to this struct is passed back to the caller of the
@@ -300,36 +325,23 @@ psf_isprint (int ch)
 
 typedef struct
 {
-	union
-	{	char		c [SF_FILENAME_LEN] ;
-		sfwchar_t	wc [SF_FILENAME_LEN] ;
-	} path ;
-
-	union
-	{	char		c [SF_FILENAME_LEN] ;
-		sfwchar_t	wc [SF_FILENAME_LEN] ;
-	} dir ;
-
-	union
-	{	char		c [SF_FILENAME_LEN / 4] ;
-		sfwchar_t	wc [SF_FILENAME_LEN / 4] ;
-	} name ;
+	char	path [SF_FILENAME_LEN] ;
+	char	dir [SF_FILENAME_LEN] ;
+	char	name [SF_FILENAME_LEN / 4] ;
 
 #if USE_WINDOWS_API
 	/*
 	**	These fields can only be used in src/file_io.c.
 	**	They are basically the same as a windows file HANDLE.
 	*/
-	void 			*handle, *hsaved ;
-
-	int				use_wchar ;
+	void 	*handle, *hsaved ;
 #else
 	/* These fields can only be used in src/file_io.c. */
-	int 			filedes, savedes ;
+	int 	filedes, savedes ;
 #endif
 
-	int				do_not_close_descriptor ;
-	int				mode ;			/* Open mode : SFM_READ, SFM_WRITE or SFM_RDWR. */
+	int		do_not_close_descriptor ;
+	int		mode ;			/* Open mode : SFM_READ, SFM_WRITE or SFM_RDWR. */
 } PSF_FILE ;
 
 
@@ -353,13 +365,6 @@ typedef union
 
 typedef struct sf_private_tag
 {
-	/* Canary in a coal mine. */
-	union
-	{	/* Place a double here to encourage double alignment. */
-		double d [2] ;
-		char c [16] ;
-		} canary ;
-
 	PSF_FILE		file, rsrc ;
 
 	char			syserr		[SF_SYSERR_LEN] ;
@@ -372,7 +377,12 @@ typedef struct sf_private_tag
 		int				indx ;
 	} parselog ;
 
-	unsigned char	header		[SF_HEADER_LEN] ; /* Must be unsigned */
+
+	struct
+	{	unsigned char	* ptr ;
+		sf_count_t		indx, end, len ;
+	} header ;
+
 	int				rwf_endian ;	/* Header endian-ness flag. */
 
 	/* Storage and housekeeping data for adding/reading strings from
@@ -391,10 +401,6 @@ typedef struct sf_private_tag
 
 	unsigned		unique_id ;
 
-	/* Index variables for maintaining parselog and header above. */
-	int				headindex, headend ;
-	int				has_text ;
-
 	int				error ;
 
 	int				endian ;		/* File endianness : SF_ENDIAN_LITTLE or SF_ENDIAN_BIG. */
@@ -421,6 +427,9 @@ typedef struct sf_private_tag
 	int				have_written ;	/* Has a single write been done to the file? */
 	PEAK_INFO		*peak_info ;
 
+	/* Cue Marker Info */
+	SF_CUES		*cues ;
+
 	/* Loop Info */
 	SF_LOOP_INFO	*loop_info ;
 	SF_INSTRUMENT	*instrument ;
@@ -511,6 +520,10 @@ typedef struct sf_private_tag
 	SF_CHUNK_ITERATOR *	(*next_chunk_iterator)	(struct sf_private_tag*, SF_CHUNK_ITERATOR * iterator) ;
 	int					(*get_chunk_size)	(struct sf_private_tag*, const SF_CHUNK_ITERATOR * iterator, SF_CHUNK_INFO * chunk_info) ;
 	int					(*get_chunk_data)	(struct sf_private_tag*, const SF_CHUNK_ITERATOR * iterator, SF_CHUNK_INFO * chunk_info) ;
+
+	int cpu_flags ;
+
+	ID3V2_HEADER_INFO	id3_header ;
 } SF_PRIVATE ;
 
 
@@ -538,7 +551,6 @@ enum
 	SFE_UNIMPLEMENTED,
 	SFE_BAD_READ_ALIGN,
 	SFE_BAD_WRITE_ALIGN,
-	SFE_UNKNOWN_FORMAT,
 	SFE_NOT_READMODE,
 	SFE_NOT_WRITEMODE,
 	SFE_BAD_MODE_RW,
@@ -553,6 +565,7 @@ enum
 	SFE_BAD_ENDIAN,
 	SFE_CHANNEL_COUNT_ZERO,
 	SFE_CHANNEL_COUNT,
+	SFE_CHANNEL_COUNT_BAD,
 
 	SFE_BAD_VIRTUAL_IO,
 
@@ -603,6 +616,7 @@ enum
 	SFE_WAV_GSM610_FORMAT,
 	SFE_WAV_UNKNOWN_CHUNK,
 	SFE_WAV_WVPK_DATA,
+	SFE_WAV_NMS_FORMAT,
 
 	SFE_AIFF_NO_FORM,
 	SFE_AIFF_AIFF_NO_FORM,
@@ -677,6 +691,7 @@ enum
 
 	SFE_DWVW_BAD_BITWIDTH,
 	SFE_G72X_NOT_MONO,
+	SFE_NMS_ADPCM_NOT_MONO,
 
 	SFE_XI_BAD_HEADER,
 	SFE_XI_EXCESS_SAMPLES,
@@ -700,6 +715,7 @@ enum
 	SFE_FLAC_INIT_DECODER,
 	SFE_FLAC_LOST_SYNC,
 	SFE_FLAC_BAD_SAMPLE_RATE,
+	SFE_FLAC_CHANNEL_COUNT_CHANGED,
 	SFE_FLAC_UNKOWN_ERROR,
 
 	SFE_WVE_NOT_WVE,
@@ -708,6 +724,9 @@ enum
 	SFE_VORBIS_ENCODER_BUG,
 
 	SFE_RF64_NOT_RF64,
+	SFE_RF64_PEAK_B4_FMT,
+	SFE_RF64_NO_DATA,
+
 	SFE_BAD_CHUNK_PTR,
 	SFE_UNKNOWN_CHUNK,
 	SFE_BAD_CHUNK_FORMAT,
@@ -715,10 +734,27 @@ enum
 	SFE_BAD_CHUNK_DATA_PTR,
 	SFE_ALAC_FAIL_TMPFILE,
 	SFE_FILENAME_TOO_LONG,
+	SFE_NEGATIVE_RW_LEN,
+
+	SFE_OPUS_BAD_SAMPLERATE,
+
+	SFE_CAF_NOT_CAF,
+	SFE_CAF_NO_DESC,
+	SFE_CAF_BAD_PEAK,
+
+	SFE_AVR_NOT_AVR,
+	SFE_AVR_BAD_REZ_SIGN,
+
+	SFE_MPC_NO_MARKER,
+
+	SFE_MPEG_BAD_SAMPLERATE,
 
 	SFE_MAX_ERROR			/* This must be last in list. */
 } ;
 
+/* Allocate and initialize the SF_PRIVATE struct. */
+SF_PRIVATE * psf_allocate (void) ;
+
 int subformat_to_bytewidth (int format) ;
 int s_bitwidth_to_subformat (int bits) ;
 int u_bitwidth_to_subformat (int bits) ;
@@ -828,6 +864,8 @@ int psf_fclose (SF_PRIVATE *psf) ;
 int psf_open_rsrc (SF_PRIVATE *psf) ;
 int psf_close_rsrc (SF_PRIVATE *psf) ;
 
+int	psf_copy_filename (SF_PRIVATE *psf, const char *path) ;
+
 /*
 void psf_fclearerr (SF_PRIVATE *psf) ;
 int psf_ferror (SF_PRIVATE *psf) ;
@@ -866,17 +904,15 @@ int		ogg_pcm_open	(SF_PRIVATE *psf) ;
 int		ogg_opus_open	(SF_PRIVATE *psf) ;
 int		ogg_open	(SF_PRIVATE *psf) ;
 
+int		mpeg_open	(SF_PRIVATE *psf) ;
 
 /* In progress. Do not currently work. */
 
-int		mpeg_open	(SF_PRIVATE *psf) ;
 int		rx2_open	(SF_PRIVATE *psf) ;
 int		txw_open	(SF_PRIVATE *psf) ;
 int		wve_open	(SF_PRIVATE *psf) ;
 int		dwd_open	(SF_PRIVATE *psf) ;
 
-int		macbinary3_open (SF_PRIVATE *psf) ;
-
 /*------------------------------------------------------------------------------------
 **	Init functions for a number of common data encodings.
 */
@@ -888,15 +924,17 @@ int		float32_init	(SF_PRIVATE *psf) ;
 int		double64_init	(SF_PRIVATE *psf) ;
 int		dwvw_init		(SF_PRIVATE *psf, int bitwidth) ;
 int		gsm610_init		(SF_PRIVATE *psf) ;
+int		nms_adpcm_init	(SF_PRIVATE *psf) ;
 int		vox_adpcm_init	(SF_PRIVATE *psf) ;
 int		flac_init		(SF_PRIVATE *psf) ;
 int		g72x_init 		(SF_PRIVATE * psf) ;
 int		alac_init		(SF_PRIVATE *psf, const ALAC_DECODER_INFO * info) ;
+int		mpeg_init		(SF_PRIVATE *psf, int bitrate_mode, int write_metadata) ;
 
 int 	dither_init		(SF_PRIVATE *psf, int mode) ;
 
-int		wav_w64_ima_init (SF_PRIVATE *psf, int blockalign, int samplesperblock) ;
-int		wav_w64_msadpcm_init (SF_PRIVATE *psf, int blockalign, int samplesperblock) ;
+int		wavlike_ima_init (SF_PRIVATE *psf, int blockalign, int samplesperblock) ;
+int		wavlike_msadpcm_init (SF_PRIVATE *psf, int blockalign, int samplesperblock) ;
 
 int		aiff_ima_init (SF_PRIVATE *psf, int blockalign, int samplesperblock) ;
 
@@ -917,17 +955,6 @@ int		psf_find_read_chunk_iterator (const READ_CHUNKS * pchk, const SF_CHUNK_ITER
 
 int		psf_find_write_chunk (WRITE_CHUNKS * pchk, const char * marker) ;
 
-static inline int
-fourcc_to_marker (const SF_CHUNK_INFO * chunk_info)
-{	const unsigned char * cptr ;
-
-	if (chunk_info->id_size != 4)
-		return 0 ;
-
-	cptr = (const unsigned char *) chunk_info->id ;
-	return (cptr [3] << 24) + (cptr [2] << 16) + (cptr [1] << 8) + cptr [0] ;
-} /* fourcc_to_marker */
-
 /*------------------------------------------------------------------------------------
 ** Functions that work like OpenBSD's strlcpy/strlcat to replace strncpy/strncat.
 **
@@ -940,17 +967,30 @@ fourcc_to_marker (const SF_CHUNK_INFO * chunk_info)
 ** compiler errors if code is carelessly converted from one to the other.
 */
 
-static inline void
-psf_strlcat (char *dest, size_t n, const char *src)
-{	strncat (dest, src, n - strlen (dest) - 1) ;
-	dest [n - 1] = 0 ;
-} /* psf_strlcat */
+void psf_strlcat (char *dest, size_t n, const char *src) ;
+void psf_strlcpy (char *dest, size_t n, const char *src) ;
 
-static inline void
-psf_strlcpy (char *dest, size_t n, const char *src)
-{	strncpy (dest, src, n - 1) ;
-	dest [n - 1] = 0 ;
-} /* psf_strlcpy */
+/*------------------------------------------------------------------------------------
+** SIMD optimized math functions.
+*/
+
+static inline int psf_lrintf (float x)
+{
+	#ifdef USE_SSE2
+ 		return _mm_cvtss_si32 (_mm_load_ss (&x)) ;
+	#else
+		return lrintf (x) ;
+	#endif
+} /* psf_lrintf */
+
+static inline int psf_lrint (double x)
+{
+	#ifdef USE_SSE2
+ 		return _mm_cvtsd_si32 (_mm_load_sd (&x)) ;
+	#else
+		return lrint (x) ;
+	#endif
+} /* psf_lrintf */
 
 /*------------------------------------------------------------------------------------
 ** Other helper functions.
@@ -958,6 +998,10 @@ psf_strlcpy (char *dest, size_t n, const char *src)
 
 void	*psf_memset (void *s, int c, sf_count_t n) ;
 
+SF_CUES * psf_cues_dup (const void * ptr, size_t datasize) ;
+SF_CUES * psf_cues_alloc (uint32_t cue_count) ;
+void psf_get_cues (SF_PRIVATE * psf, void * data, size_t datasize) ;
+
 SF_INSTRUMENT * psf_instrument_alloc (void) ;
 
 void	psf_sanitize_string (char * cptr, int len) ;
@@ -981,6 +1025,7 @@ typedef struct
 
 int audio_detect (SF_PRIVATE * psf, AUDIO_DETECT *ad, const unsigned char * data, int datalen) ;
 int id3_skip (SF_PRIVATE * psf) ;
+const char *id3_lookup_v1_genre (int number) ;
 
 void	alac_get_desc_chunk_items (int subformat, uint32_t *fmt_flags, uint32_t *frames_per_packet) ;
 
