@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2002-2015 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2002-2018 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -54,7 +54,7 @@
 #define	M_PI		3.14159265358979323846264338
 #endif
 
-#define	LOG_BUFFER_SIZE		2048
+#define	LOG_BUFFER_SIZE		4096
 
 /*
 **	Neat solution to the Win32/OS2 binary file flage requirement.
@@ -109,6 +109,7 @@ create_short_sndfile (const char *filename, int format, int channels)
 	SNDFILE *file ;
 	SF_INFO sfinfo ;
 
+	memset (&sfinfo, 0, sizeof (sfinfo)) ;
 	sfinfo.samplerate = 44100 ;
 	sfinfo.channels = channels ;
 	sfinfo.format = format ;
@@ -452,6 +453,18 @@ dump_log_buffer (SNDFILE *file)
 	return ;
 } /* dump_log_buffer */
 
+void
+test_sf_format_or_die (const SF_INFO *info, int line_num)
+{	int res ;
+
+	if ((res = sf_format_check (info)) != 1)
+	{	printf ("\n\nLine %d : sf_format_check returned error (%d)\n\n", line_num, res) ;
+		exit (1) ;
+		} ;
+
+	return ;
+} /* test_sf_format_or_die */
+
 SNDFILE *
 test_open_file_or_die (const char *filename, int mode, SF_INFO *sfinfo, int allow_fd, int line_num)
 {	static int count = 0 ;
@@ -1023,6 +1036,17 @@ delete_file (int format, const char *filename)
 	unlink (rsrc_name) ;
 } /* delete_file */
 
+int
+truncate_file_to_zero (const char * fname)
+{	FILE * file ;
+
+	if ((file = fopen (fname, "w")) == NULL)
+		return errno ;
+	fclose (file) ;
+
+	return 0 ;
+} /* truncate_file_to_zero */
+
 static int allowed_open_files = -1 ;
 
 void
@@ -1074,6 +1098,15 @@ check_open_file_count_or_die (int lineno)
 #endif
 } /* check_open_file_count_or_die */
 
+void
+get_unique_test_name (const char ** filename, const char * test)
+{	static char	buffer [1024] ;
+
+	snprintf (buffer, sizeof (buffer), "%s_%s", test, *filename) ;
+
+	*filename = buffer ;
+} /* get_unique_test_name */
+
 void
 write_mono_file (const char * filename, int format, int srate, float * output, int len)
 {	SNDFILE * file ;
